/*
 *  Copyright(c)2003 Forward Computing and Control Pty. Ltd.
 *  ACN 003 669 994, NSW, Australia      All rights Reserved
 *
 *  Written by Dr. M.P. Ford

Logging package
Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source of this package, with or without
modification, is NOT permitted.

Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.

2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the 
   distribution, a statement that the software has been modified and
   identify the person who was responsable for the modifications.  

3. Neither the name of the author nor Forward Computing and Control may be 
   used to endorse or promote products derived from this software
   without specific prior written permission.

4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment: 
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd. 
    (http://www.forward.com.au)."


THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 
 *
 */
package au.com.forward.logging;

import java.util.*;
import java.util.logging.*;
import java.text.MessageFormat;

/**
 *  This class provides a robust simple formatter.
 *
 *@author     Matthew Ford
 *@created    October 9, 2003
 */
public class LoggingSimpleFormatter extends RobustFormatter {

	/**
	 *  the string to use for new lines.
	 *  <br>Defaults to \n if not specified
	 */
	private static String newLine = "\n";

	static {
		try {
			newLine = (String) java.security.AccessController.doPrivileged(
					new sun.security.action.GetPropertyAction("line.separator"));
		} catch (Throwable t) {
			// print an error
			// LogStdStreams initialized in the main application will redirect this to a file.
			System.err.println("Error getting system line separator for Logging will use \\n\n" + toString(t));
		}
	}

	/**
	 *  the date object for the time/date output
	 */
	Date date = new Date();
	/**
	 *  the date time format
	 */
	private final static String format = "{0,date} {0,time}";

	/**
	 *  the formatter for the date and time
	 */
	private MessageFormat formatter = null;

	/**
	 *  the arguments for the date time formatter
	 */
	private Object args[] = new Object[1];


	//~ Methods ..................................................................

	/**
	 *  Returns the newLine setting for this class
	 *  <br>Use system newLine setting.
	 *
	 *@return    the string to use for new lines.
	 */
	protected String newLineString() {
		return newLine;
	}


	/**
	 *  Format the given LogRecord. Don't allow any errors to be thrown here
	 *
	 *@param  record  the log record to be formatted.
	 *@return         a formatted log record
	 */
	public synchronized String format(LogRecord record) {
		try {
			StringBuffer sb = new StringBuffer();
			StringBuffer text = new StringBuffer();
			if (formatter == null) {
				formatter = new MessageFormat(format);
			}
			try {
				date.setTime(record.getMillis());
				args[0] = date;
				formatter.format(args, text, null);
			} catch (Throwable t1) {
				text.append("Error formatting record date and time" + newLineString() + toString(t1));
			}
			sb.append(text);
			sb.append(" ");
			try {
				if (record.getSourceClassName() != null) {
					sb.append(record.getSourceClassName());
				} else {
					sb.append(record.getLoggerName());
				}
			} catch (Throwable t2) {
				sb.append("Error getting class name" + newLineString() + toString(t2));
			}
			try {
				if (record.getSourceMethodName() != null) {
					sb.append(" ");
					sb.append(record.getSourceMethodName());
				}
			} catch (Throwable t3) {
				sb.append("Error getting method name" + newLineString() + toString(t3));
			}

			sb.append(newLineString());
			try {
				sb.append(record.getLevel().getLocalizedName());
			} catch (Throwable t4) {
				sb.append("Error getting localized level name" + newLineString() + toString(t4));
			}
			sb.append(": ");
			try {
				sb.append(formatMessage(record));
			} catch (Throwable t5) {
				sb.append("Error formatting record message" + newLineString() + toString(t5));
			}
			sb.append(newLineString());
			try {
				Throwable t = record.getThrown();
				if (t != null) {
					sb.append(toString(t));
				}
			} catch (Throwable t6) {
				sb.append("Error getting record exception thrown" + newLineString() + toString(t6));
			}
			sb.append(newLineString());
			return sb.toString();
		} catch (Throwable t) {
			return ("Unexpected error caught while trying to log a record" + newLineString() + toString(t));
		}
	}

}

